require 'kpeg/compiled_parser'

class APL::Parser < KPeg::CompiledParser


  def success?
    @pos == @string_size
  end

  def failure?
    !success?
  end


  # :stopdoc:

  # space = /\s/
  def _space
    _tmp = scan(/\A(?-mix:\s)/)
    set_failed_rule :_space unless _tmp
    return _tmp
  end

  # - = space*
  def __hyphen_
    while true
      _tmp = apply(:_space)
      break unless _tmp
    end
    _tmp = true
    set_failed_rule :__hyphen_ unless _tmp
    return _tmp
  end

  # DIGIT = /[0-9]/
  def _DIGIT
    _tmp = scan(/\A(?-mix:[0-9])/)
    set_failed_rule :_DIGIT unless _tmp
    return _tmp
  end

  # digits = < DIGIT+ > { text }
  def _digits

    _save = self.pos
    while true # sequence
      _text_start = self.pos
      _save1 = self.pos
      _tmp = apply(:_DIGIT)
      if _tmp
        while true
          _tmp = apply(:_DIGIT)
          break unless _tmp
        end
        _tmp = true
      else
        self.pos = _save1
      end
      if _tmp
        text = get_text(_text_start)
      end
      unless _tmp
        self.pos = _save
        break
      end
      @result = begin;  text ; end
      _tmp = true
      unless _tmp
        self.pos = _save
      end
      break
    end # end sequence

    set_failed_rule :_digits unless _tmp
    return _tmp
  end

  # exponent_prefix = /e/i
  def _exponent_prefix
    _tmp = scan(/\A(?i-mx:e)/)
    set_failed_rule :_exponent_prefix unless _tmp
    return _tmp
  end

  # negative_sign = "¯" { -1 }
  def _negative_sign

    _save = self.pos
    while true # sequence
      _tmp = match_string("\u{af}")
      unless _tmp
        self.pos = _save
        break
      end
      @result = begin;  -1 ; end
      _tmp = true
      unless _tmp
        self.pos = _save
      end
      break
    end # end sequence

    set_failed_rule :_negative_sign unless _tmp
    return _tmp
  end

  # optional_negative = negative_sign?:negative { negative || 1 }
  def _optional_negative

    _save = self.pos
    while true # sequence
      _save1 = self.pos
      _tmp = apply(:_negative_sign)
      @result = nil unless _tmp
      unless _tmp
        _tmp = true
        self.pos = _save1
      end
      negative = @result
      unless _tmp
        self.pos = _save
        break
      end
      @result = begin;  negative || 1 ; end
      _tmp = true
      unless _tmp
        self.pos = _save
      end
      break
    end # end sequence

    set_failed_rule :_optional_negative unless _tmp
    return _tmp
  end

  # radix = < "." > { text }
  def _radix

    _save = self.pos
    while true # sequence
      _text_start = self.pos
      _tmp = match_string(".")
      if _tmp
        text = get_text(_text_start)
      end
      unless _tmp
        self.pos = _save
        break
      end
      @result = begin;  text ; end
      _tmp = true
      unless _tmp
        self.pos = _save
      end
      break
    end # end sequence

    set_failed_rule :_radix unless _tmp
    return _tmp
  end

  # function = < /[-+×÷]/ > { text }
  def _function

    _save = self.pos
    while true # sequence
      _text_start = self.pos
      _tmp = scan(/\A(?-mix:[-+×÷])/)
      if _tmp
        text = get_text(_text_start)
      end
      unless _tmp
        self.pos = _save
        break
      end
      @result = begin;  text ; end
      _tmp = true
      unless _tmp
        self.pos = _save
      end
      break
    end # end sequence

    set_failed_rule :_function unless _tmp
    return _tmp
  end

  # root = - expression
  def _root

    _save = self.pos
    while true # sequence
      _tmp = apply(:__hyphen_)
      unless _tmp
        self.pos = _save
        break
      end
      _tmp = apply(:_expression)
      unless _tmp
        self.pos = _save
      end
      break
    end # end sequence

    set_failed_rule :_root unless _tmp
    return _tmp
  end

  # expression = (function_call | term):expression - { expression }
  def _expression

    _save = self.pos
    while true # sequence

      _save1 = self.pos
      while true # choice
        _tmp = apply(:_function_call)
        break if _tmp
        self.pos = _save1
        _tmp = apply(:_term)
        break if _tmp
        self.pos = _save1
        break
      end # end choice

      expression = @result
      unless _tmp
        self.pos = _save
        break
      end
      _tmp = apply(:__hyphen_)
      unless _tmp
        self.pos = _save
        break
      end
      @result = begin;  expression ; end
      _tmp = true
      unless _tmp
        self.pos = _save
      end
      break
    end # end sequence

    set_failed_rule :_expression unless _tmp
    return _tmp
  end

  # function_call = term?:x - function:op - expression:y { APL::AST::Function.new op: op.to_sym, x: x, y: y }
  def _function_call

    _save = self.pos
    while true # sequence
      _save1 = self.pos
      _tmp = apply(:_term)
      @result = nil unless _tmp
      unless _tmp
        _tmp = true
        self.pos = _save1
      end
      x = @result
      unless _tmp
        self.pos = _save
        break
      end
      _tmp = apply(:__hyphen_)
      unless _tmp
        self.pos = _save
        break
      end
      _tmp = apply(:_function)
      op = @result
      unless _tmp
        self.pos = _save
        break
      end
      _tmp = apply(:__hyphen_)
      unless _tmp
        self.pos = _save
        break
      end
      _tmp = apply(:_expression)
      y = @result
      unless _tmp
        self.pos = _save
        break
      end
      @result = begin;  APL::AST::Function.new op: op.to_sym, x: x, y: y ; end
      _tmp = true
      unless _tmp
        self.pos = _save
      end
      break
    end # end sequence

    set_failed_rule :_function_call unless _tmp
    return _tmp
  end

  # term = (primitive_value:value | "(" - expression:value - ")" { value })
  def _term

    _save = self.pos
    while true # choice
      _tmp = apply(:_primitive_value)
      value = @result
      break if _tmp
      self.pos = _save

      _save1 = self.pos
      while true # sequence
        _tmp = match_string("(")
        unless _tmp
          self.pos = _save1
          break
        end
        _tmp = apply(:__hyphen_)
        unless _tmp
          self.pos = _save1
          break
        end
        _tmp = apply(:_expression)
        value = @result
        unless _tmp
          self.pos = _save1
          break
        end
        _tmp = apply(:__hyphen_)
        unless _tmp
          self.pos = _save1
          break
        end
        _tmp = match_string(")")
        unless _tmp
          self.pos = _save1
          break
        end
        @result = begin;  value ; end
        _tmp = true
        unless _tmp
          self.pos = _save1
        end
        break
      end # end sequence

      break if _tmp
      self.pos = _save
      break
    end # end choice

    set_failed_rule :_term unless _tmp
    return _tmp
  end

  # primitive_value = (vector | number):value { value }
  def _primitive_value

    _save = self.pos
    while true # sequence

      _save1 = self.pos
      while true # choice
        _tmp = apply(:_vector)
        break if _tmp
        self.pos = _save1
        _tmp = apply(:_number)
        break if _tmp
        self.pos = _save1
        break
      end # end choice

      value = @result
      unless _tmp
        self.pos = _save
        break
      end
      @result = begin;  value ; end
      _tmp = true
      unless _tmp
        self.pos = _save
      end
      break
    end # end sequence

    set_failed_rule :_primitive_value unless _tmp
    return _tmp
  end

  # vector = number:first primitive_value:rest - { Array(first) + Array(rest) }
  def _vector

    _save = self.pos
    while true # sequence
      _tmp = apply(:_number)
      first = @result
      unless _tmp
        self.pos = _save
        break
      end
      _tmp = apply(:_primitive_value)
      rest = @result
      unless _tmp
        self.pos = _save
        break
      end
      _tmp = apply(:__hyphen_)
      unless _tmp
        self.pos = _save
        break
      end
      @result = begin;  Array(first) + Array(rest) ; end
      _tmp = true
      unless _tmp
        self.pos = _save
      end
      break
    end # end sequence

    set_failed_rule :_vector unless _tmp
    return _tmp
  end

  # number = optional_negative:negative nonnegative_integer:integer decimal?:decimal exponent?:exponent - { (integer + (decimal ? decimal.to_f : 0)) * negative * (exponent || 1) }
  def _number

    _save = self.pos
    while true # sequence
      _tmp = apply(:_optional_negative)
      negative = @result
      unless _tmp
        self.pos = _save
        break
      end
      _tmp = apply(:_nonnegative_integer)
      integer = @result
      unless _tmp
        self.pos = _save
        break
      end
      _save1 = self.pos
      _tmp = apply(:_decimal)
      @result = nil unless _tmp
      unless _tmp
        _tmp = true
        self.pos = _save1
      end
      decimal = @result
      unless _tmp
        self.pos = _save
        break
      end
      _save2 = self.pos
      _tmp = apply(:_exponent)
      @result = nil unless _tmp
      unless _tmp
        _tmp = true
        self.pos = _save2
      end
      exponent = @result
      unless _tmp
        self.pos = _save
        break
      end
      _tmp = apply(:__hyphen_)
      unless _tmp
        self.pos = _save
        break
      end
      @result = begin;  (integer + (decimal ? decimal.to_f : 0)) * negative * (exponent || 1) ; end
      _tmp = true
      unless _tmp
        self.pos = _save
      end
      break
    end # end sequence

    set_failed_rule :_number unless _tmp
    return _tmp
  end

  # integer = optional_negative:negative nonnegative_integer:integer { integer * negative }
  def _integer

    _save = self.pos
    while true # sequence
      _tmp = apply(:_optional_negative)
      negative = @result
      unless _tmp
        self.pos = _save
        break
      end
      _tmp = apply(:_nonnegative_integer)
      integer = @result
      unless _tmp
        self.pos = _save
        break
      end
      @result = begin;  integer * negative ; end
      _tmp = true
      unless _tmp
        self.pos = _save
      end
      break
    end # end sequence

    set_failed_rule :_integer unless _tmp
    return _tmp
  end

  # nonnegative_integer = digits:num { num.to_i }
  def _nonnegative_integer

    _save = self.pos
    while true # sequence
      _tmp = apply(:_digits)
      num = @result
      unless _tmp
        self.pos = _save
        break
      end
      @result = begin;  num.to_i ; end
      _tmp = true
      unless _tmp
        self.pos = _save
      end
      break
    end # end sequence

    set_failed_rule :_nonnegative_integer unless _tmp
    return _tmp
  end

  # decimal = < radix digits > { text }
  def _decimal

    _save = self.pos
    while true # sequence
      _text_start = self.pos

      _save1 = self.pos
      while true # sequence
        _tmp = apply(:_radix)
        unless _tmp
          self.pos = _save1
          break
        end
        _tmp = apply(:_digits)
        unless _tmp
          self.pos = _save1
        end
        break
      end # end sequence

      if _tmp
        text = get_text(_text_start)
      end
      unless _tmp
        self.pos = _save
        break
      end
      @result = begin;  text ; end
      _tmp = true
      unless _tmp
        self.pos = _save
      end
      break
    end # end sequence

    set_failed_rule :_decimal unless _tmp
    return _tmp
  end

  # exponent = exponent_prefix integer:exponent { 10 ** exponent }
  def _exponent

    _save = self.pos
    while true # sequence
      _tmp = apply(:_exponent_prefix)
      unless _tmp
        self.pos = _save
        break
      end
      _tmp = apply(:_integer)
      exponent = @result
      unless _tmp
        self.pos = _save
        break
      end
      @result = begin;  10 ** exponent ; end
      _tmp = true
      unless _tmp
        self.pos = _save
      end
      break
    end # end sequence

    set_failed_rule :_exponent unless _tmp
    return _tmp
  end

  Rules = {}
  Rules[:_space] = rule_info("space", "/\\s/")
  Rules[:__hyphen_] = rule_info("-", "space*")
  Rules[:_DIGIT] = rule_info("DIGIT", "/[0-9]/")
  Rules[:_digits] = rule_info("digits", "< DIGIT+ > { text }")
  Rules[:_exponent_prefix] = rule_info("exponent_prefix", "/e/i")
  Rules[:_negative_sign] = rule_info("negative_sign", "\"¯\" { -1 }")
  Rules[:_optional_negative] = rule_info("optional_negative", "negative_sign?:negative { negative || 1 }")
  Rules[:_radix] = rule_info("radix", "< \".\" > { text }")
  Rules[:_function] = rule_info("function", "< /[-+×÷]/ > { text }")
  Rules[:_root] = rule_info("root", "- expression")
  Rules[:_expression] = rule_info("expression", "(function_call | term):expression - { expression }")
  Rules[:_function_call] = rule_info("function_call", "term?:x - function:op - expression:y { APL::AST::Function.new op: op.to_sym, x: x, y: y }")
  Rules[:_term] = rule_info("term", "(primitive_value:value | \"(\" - expression:value - \")\" { value })")
  Rules[:_primitive_value] = rule_info("primitive_value", "(vector | number):value { value }")
  Rules[:_vector] = rule_info("vector", "number:first primitive_value:rest - { Array(first) + Array(rest) }")
  Rules[:_number] = rule_info("number", "optional_negative:negative nonnegative_integer:integer decimal?:decimal exponent?:exponent - { (integer + (decimal ? decimal.to_f : 0)) * negative * (exponent || 1) }")
  Rules[:_integer] = rule_info("integer", "optional_negative:negative nonnegative_integer:integer { integer * negative }")
  Rules[:_nonnegative_integer] = rule_info("nonnegative_integer", "digits:num { num.to_i }")
  Rules[:_decimal] = rule_info("decimal", "< radix digits > { text }")
  Rules[:_exponent] = rule_info("exponent", "exponent_prefix integer:exponent { 10 ** exponent }")
  # :startdoc:
end
